(define (make-easy-hash)
  (let* ((h (make-hash-table))
         (acc 
          (lambda (x . y)
            (case x
              ('get (hash-table-ref h (car y)))
              ('set (hash-table-set! h (car y) (cadr y)))
              ('has (hash-table-exists? h (car y)))
              ('del (hash-table-delete! h (car y)))
              ('cons (hash-cons-key h (car y) (cadr y)))
              ('alist (hash-table->alist h))
              (else (error (list "Don't know what this is: " x)))))))
    acc))

(define (make-easy-2-key-hash)
  (define (mk-key key1 key2)
    (s+ (->string key1) "->" (->string key2)))
  (define (get-key1 y) (car y))
  (define (get-key2 y) (cadr y))
  (define (get-key y) (mk-key (get-key1 y) (get-key2 y)))
  (define (get-arg y) (caddr y))
  (let* ((h (make-hash-table equal?))
         (acc 
          (lambda (x . y)
            (case x
              ('get (hash-table-ref h (get-key y)))
              ('set (hash-table-set! h (get-key y) (get-arg y)))
              ('has (hash-table-exists? h (get-key y)))
              ('del (hash-table-delete! h (get-key y)))
              ('cons (hash-cons-key h (get-key y) (get-arg y)))
              ('alist (hash-table->alist h))
              (else (error (list "Don't know what this is: " x)))))))
    acc))

(define (hash-cons-key hash-table key value)
  (hash-table-update!/default hash-table key (lambda (x) (cons value x)) '()))

(define (make-easy-n-key-hash)
  (define (mk-key keys) keys)
   ; (string-join (map ->string keys) "->"))
  (define (get-key y) (mk-key (car y)))
  (define (get-arg y) (cadr y))
  (define (get-arg-no-key y) (car y))
  (let* ((h (make-hash-table equal?))
         (acc 
          (lambda (x . y)
            (case x
              ('get (hash-table-ref h (get-key y)))
              ('set (hash-table-set! h (get-key y) (get-arg y)))
              ('has (hash-table-exists? h (get-key y)))
              ('del (hash-table-delete! h (get-key y)))
              ('cons (hash-cons-key h (get-key y) (get-arg y)))
              ('alist (hash-table->alist h))
              ('get-nth-keys (delete-duplicates! (map (lambda (x) (list-ref x (get-arg-no-key y))) (hash-table-keys h))))
              (else (error (list "Don't know what this is: " x)))))))
    acc))


(define (test-easy-2-hash)
  (define h (make-easy-2-key-hash))
  (h 'set "abram" "loves" "lixin")
  (h 'set "abram" "hates" "work")
  (h 'set "lixin" "loves" "abram")
  (list
   (h 'get "abram" "loves")
   (h 'get "lixin" "loves")
   (h 'get "abram" "hates")))

(define (test-easy-n-hash)
  (define h (make-easy-n-key-hash))
  (h 'set '("abram" "loves") "lixin")
  (h 'set '("abram" "hates") "work")
  (h 'set '("lixin" "loves") "abram")
  (list
   (h 'get '("abram" "loves"))
   (h 'get '("lixin" "loves"))
   (h 'get '("abram" "hates"))))
   

